#!/usr/bin/perl

# Copyright 2016 Michael Schlenstedt, michael@loxberry.de
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

##########################################################################
# Modules
##########################################################################

use CGI::Carp qw(fatalsToBrowser);
use CGI qw/:standard/;
use Config::Simple;
use File::HomeDir;
use Cwd 'abs_path';
use warnings;
use strict;
no strict "refs";    # we need it for template system

##########################################################################
# Variables
##########################################################################

our $cfg;
our $pcfg;
our $phrase;
our $namef;
our $value;
our %query;
our $lang;
our $template_title;
our $help;
our @help;
our $helptext;
our $helplink;
our $installfolder;
our $languagefile;
our $version;
our $error;
our $saveformdata = 0;
our $output;
our $message;
our $nexturl;
our $do = "form";
my $home = File::HomeDir->my_home;
our $psubfolder;
our $pname;
our $languagefileplugin;
our $phraseplugin;
our $header_already_sent = 0;

our $meter_active_1;
our $meter_active_1_selected;
our $id_1;
our $time_1;
our $meter_active_2;
our $meter_active_2_selected;
our $id_2;
our $time_2;
our $meter_active_3;
our $meter_active_3_selected;
our $id_3;
our $time_3;
our $meter_active_4;
our $meter_active_4_selected;
our $id_4;
our $time_4;
our $meter_active_5;
our $meter_active_5_selected;
our $id_5;
our $time_5;
our $meter_active_6;
our $meter_active_6_selected;
our $id_6;
our $time_6;
our $meter_active_7;
our $meter_active_7_selected;
our $id_7;
our $time_7;
our $meter_active_8;
our $meter_active_8_selected;
our $id_8;
our $time_8;
our $meter_active_9;
our $meter_active_9_selected;
our $id_9;
our $time_9;
our $meter_active_10;
our $meter_active_10_selected;
our $id_10;
our $time_10;
our $meter_active_11;
our $meter_active_11_selected;
our $id_11;
our $time_11;
our $meter_active_12;
our $meter_active_12_selected;
our $id_12;
our $time_12;
our $meter_active_13;
our $meter_active_13_selected;
our $id_13;
our $time_13;
our $meter_active_14;
our $meter_active_14_selected;
our $id_14;
our $time_14;
our $meter_active_15;
our $meter_active_15_selected;
our $id_15;
our $time_15;

our $rs_connect;
our $rs_connect_select;
our $rs_baud;
our $rs_baud_select;
our $httpport;

our $miniserver;
our $miniserver_select;
our $udpport;
our $debug;
our $debug_selected;

##########################################################################
# Read Settings
##########################################################################

# Version of this script
$version = "0.0.8";

# Figure out in which subfolder we are installed
$psubfolder = abs_path($0);
$psubfolder =~ s/(.*)\/(.*)\/(.*)$/$2/g;

#Read Config
$cfg           = new Config::Simple("$home/config/system/general.cfg");
$installfolder = $cfg->param("BASE.INSTALLFOLDER");
$lang          = $cfg->param("BASE.LANG");
$pcfg = new Config::Simple("$installfolder/config/plugins/$psubfolder/modbus_powermeter.cfg");

$meter_active_1 = $pcfg->param("METER_1.METER_ACTIVE");
$id_1 = $pcfg->param("METER_1.ID");
$time_1 = $pcfg->param("METER_1.TIME");
$meter_active_2 = $pcfg->param("METER_2.METER_ACTIVE");
$id_2 = $pcfg->param("METER_2.ID");
$time_2 = $pcfg->param("METER_2.TIME");
$meter_active_3 = $pcfg->param("METER_3.METER_ACTIVE");
$id_3 = $pcfg->param("METER_3.ID");
$time_3 = $pcfg->param("METER_3.TIME");
$meter_active_4 = $pcfg->param("METER_4.METER_ACTIVE");
$id_4 = $pcfg->param("METER_4.ID");
$time_4 = $pcfg->param("METER_4.TIME");
$meter_active_5 = $pcfg->param("METER_5.METER_ACTIVE");
$id_5 = $pcfg->param("METER_5.ID");
$time_5 = $pcfg->param("METER_5.TIME");
$meter_active_6 = $pcfg->param("METER_6.METER_ACTIVE");
$id_6 = $pcfg->param("METER_6.ID");
$time_6 = $pcfg->param("METER_6.TIME");
$meter_active_7 = $pcfg->param("METER_7.METER_ACTIVE");
$id_7 = $pcfg->param("METER_7.ID");
$time_7 = $pcfg->param("METER_7.TIME");
$meter_active_8 = $pcfg->param("METER_8.METER_ACTIVE");
$id_8 = $pcfg->param("METER_8.ID");
$time_8 = $pcfg->param("METER_8.TIME");
$meter_active_9 = $pcfg->param("METER_9.METER_ACTIVE");
$id_9 = $pcfg->param("METER_9.ID");
$time_9 = $pcfg->param("METER_9.TIME");
$meter_active_10 = $pcfg->param("METER_10.METER_ACTIVE");
$id_10 = $pcfg->param("METER_10.ID");
$time_10 = $pcfg->param("METER_10.TIME");
$meter_active_11 = $pcfg->param("METER_11.METER_ACTIVE");
$id_11 = $pcfg->param("METER_11.ID");
$time_11 = $pcfg->param("METER_11.TIME");
$meter_active_12 = $pcfg->param("METER_12.METER_ACTIVE");
$id_12 = $pcfg->param("METER_12.ID");
$time_12 = $pcfg->param("METER_12.TIME");
$meter_active_13 = $pcfg->param("METER_13.METER_ACTIVE");
$id_13 = $pcfg->param("METER_13.ID");
$time_13 = $pcfg->param("METER_13.TIME");
$meter_active_14 = $pcfg->param("METER_14.METER_ACTIVE");
$id_14 = $pcfg->param("METER_14.ID");
$time_14 = $pcfg->param("METER_14.TIME");
$meter_active_15 = $pcfg->param("METER_15.METER_ACTIVE");
$id_15 = $pcfg->param("METER_15.ID");
$time_15 = $pcfg->param("METER_15.TIME");

$rs_connect = $pcfg->param("RS485.CONNECT");
$rs_baud = $pcfg->param("RS485.BAUD");
$httpport = $pcfg->param("HTTP.PORT");

$miniserver = $pcfg->param("MINISERVER.MINISERVER");
$udpport = $pcfg->param("MINISERVER.UDPPORT");
$debug = $pcfg->param("DEBUG.DEBUG");

#########################################################################
# Parameter
#########################################################################

# Everything from URL
foreach ( split( /&/, $ENV{'QUERY_STRING'} ) ) {
    ( $namef, $value ) = split( /=/, $_, 2 );
    $namef =~ tr/+/ /;
    $namef =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
    $value =~ tr/+/ /;
    $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
    $query{$namef} = $value;
}

# Set parameters coming in - get over post
if ( !$query{'saveformdata'} ) {
    if ( param('saveformdata') ) {
        $saveformdata = quotemeta( param('saveformdata') );
    }
    else { $saveformdata = 0; }
}
else { $saveformdata = quotemeta( $query{'saveformdata'} ); }
if ( !$query{'lang'} ) {
    if   ( param('lang') ) { $lang = quotemeta( param('lang') ); }
    else                   { $lang = "de"; }
}
else { $lang = quotemeta( $query{'lang'} ); }
if ( !$query{'do'} ) {
    if   ( param('do') ) { $do = quotemeta( param('do') ); }
    else                 { $do = "form"; }
}
else { $do = quotemeta( $query{'do'} ); }

# Clean up saveformdata variable
$saveformdata =~ tr/0-1//cd;
$saveformdata = substr( $saveformdata, 0, 1 );

# Init Language
# Clean up lang variable
$lang =~ tr/a-z//cd;
$lang = substr( $lang, 0, 2 );

# If there's no language phrases file for choosed language, use german as default
if ( !-e "$installfolder/templates/system/$lang/language.dat" ) {
    $lang = "de";
}

# Read translations / phrases
$languagefile = "$installfolder/templates/system/$lang/language.dat";
$phrase       = new Config::Simple($languagefile);
$languagefileplugin ="$installfolder/templates/plugins/$psubfolder/$lang/language.dat";
$phraseplugin = new Config::Simple($languagefileplugin);

##########################################################################
# Main program
##########################################################################

if ($saveformdata) {
    &save;
}
else {
    &form;
}
exit;

#####################################################
#
# Subroutines
#
#####################################################

#####################################################
# Form-Sub
#####################################################

sub form {

    # Filter
    #$lon     = quotemeta($lon);
	
# Miniserver selection dropdown
for (my $i = 1; $i <= $cfg->param('BASE.MINISERVERS');$i++) {
	if ("MINISERVER$i" eq $miniserver) {
		$miniserver_select .= '<option selected value="'.$i.'">'.$cfg->param("MINISERVER$i.NAME")."</option>\n";
	} else {
		$miniserver_select .= '<option value="'.$i.'">'.$cfg->param("MINISERVER$i.NAME")."</option>\n";
	}
}

# Baud selection dropdown
if ($rs_baud eq "1") {
	$rs_baud_select = '<option selected value="1">2400</option><option value="2">9600</option>\n';
} elsif ($rs_baud eq "2") { 
	$rs_baud_select = '<option value="1">2400</option><option selected value="2">9600</option>\n';
} else { 
	$rs_baud_select = '<option selected value="1">2400</option><option value="2">9600</option>\n';
}

if ($rs_connect eq "/dev/ttyAMA0") {
	$rs_connect_select = '
	<option selected value="/dev/ttyAMA0">/dev/ttyAMA0</option>
    <option value="/dev/ttyUSB0">/dev/ttyUSB0</option>
	<option value="/dev/ttyUSB1">/dev/ttyUSB1</option>
	<option value="/dev/ttyUSB2">/dev/ttyUSB2</option>
	<option value="/dev/ttyUSB3">/dev/ttyUSB3</option>
	<option value="/dev/ttyUSB4">/dev/ttyUSB4</option>
	<option value="/dev/ttyUSB5">/dev/ttyUSB5</option>
	<option value="/dev/ttyUSB6">/dev/ttyUSB6</option>
	<option value="/dev/ttyUSB7">/dev/ttyUSB7</option>
	<option value="/dev/ttyUSB8">/dev/ttyUSB8</option>
	<option value="/dev/ttyUSB9">/dev/ttyUSB9</option>\n';
} elsif ($rs_connect eq "/dev/ttyUSB0") {
	$rs_connect_select = '
	<option value="/dev/ttyAMA0">/dev/ttyAMA0</option>
    <option selected value="/dev/ttyUSB0">/dev/ttyUSB0</option>
	<option value="/dev/ttyUSB1">/dev/ttyUSB1</option>
	<option value="/dev/ttyUSB2">/dev/ttyUSB2</option>
	<option value="/dev/ttyUSB3">/dev/ttyUSB3</option>
	<option value="/dev/ttyUSB4">/dev/ttyUSB4</option>
	<option value="/dev/ttyUSB5">/dev/ttyUSB5</option>
	<option value="/dev/ttyUSB6">/dev/ttyUSB6</option>
	<option value="/dev/ttyUSB7">/dev/ttyUSB7</option>
	<option value="/dev/ttyUSB8">/dev/ttyUSB8</option>
	<option value="/dev/ttyUSB9">/dev/ttyUSB9</option>\n';
} elsif ($rs_connect eq "/dev/ttyUSB1") {
	$rs_connect_select = '
	<option value="/dev/ttyAMA0">/dev/ttyAMA0</option>
    <option value="/dev/ttyUSB0">/dev/ttyUSB0</option>
	<option selected value="/dev/ttyUSB1">/dev/ttyUSB1</option>
	<option value="/dev/ttyUSB2">/dev/ttyUSB2</option>
	<option value="/dev/ttyUSB3">/dev/ttyUSB3</option>
	<option value="/dev/ttyUSB4">/dev/ttyUSB4</option>
	<option value="/dev/ttyUSB5">/dev/ttyUSB5</option>
	<option value="/dev/ttyUSB6">/dev/ttyUSB6</option>
	<option value="/dev/ttyUSB7">/dev/ttyUSB7</option>
	<option value="/dev/ttyUSB8">/dev/ttyUSB8</option>
	<option value="/dev/ttyUSB9">/dev/ttyUSB9</option>\n';	
} elsif ($rs_connect eq "/dev/ttyUSB2") {
	$rs_connect_select = '
	<option value="/dev/ttyAMA0">/dev/ttyAMA0</option>
    <option value="/dev/ttyUSB0">/dev/ttyUSB0</option>
	<option value="/dev/ttyUSB1">/dev/ttyUSB1</option>
	<option selected value="/dev/ttyUSB2">/dev/ttyUSB2</option>
	<option value="/dev/ttyUSB3">/dev/ttyUSB3</option>
	<option value="/dev/ttyUSB4">/dev/ttyUSB4</option>
	<option value="/dev/ttyUSB5">/dev/ttyUSB5</option>
	<option value="/dev/ttyUSB6">/dev/ttyUSB6</option>
	<option value="/dev/ttyUSB7">/dev/ttyUSB7</option>
	<option value="/dev/ttyUSB8">/dev/ttyUSB8</option>
	<option value="/dev/ttyUSB9">/dev/ttyUSB9</option>\n';	

} elsif ($rs_connect eq "/dev/ttyUSB3") {
	$rs_connect_select = '
	<option value="/dev/ttyAMA0">/dev/ttyAMA0</option>
    <option value="/dev/ttyUSB0">/dev/ttyUSB0</option>
	<option value="/dev/ttyUSB1">/dev/ttyUSB1</option>
	<option value="/dev/ttyUSB2">/dev/ttyUSB2</option>
	<option selected value="/dev/ttyUSB3">/dev/ttyUSB3</option>
	<option value="/dev/ttyUSB4">/dev/ttyUSB4</option>
	<option value="/dev/ttyUSB5">/dev/ttyUSB5</option>
	<option value="/dev/ttyUSB6">/dev/ttyUSB6</option>
	<option value="/dev/ttyUSB7">/dev/ttyUSB7</option>
	<option value="/dev/ttyUSB8">/dev/ttyUSB8</option>
	<option value="/dev/ttyUSB9">/dev/ttyUSB9</option>\n';	

} elsif ($rs_connect eq "/dev/ttyUSB4") {
	$rs_connect_select = '
	<option value="/dev/ttyAMA0">/dev/ttyAMA0</option>
    <option value="/dev/ttyUSB0">/dev/ttyUSB0</option>
	<option value="/dev/ttyUSB1">/dev/ttyUSB1</option>
	<option value="/dev/ttyUSB2">/dev/ttyUSB2</option>
	<option value="/dev/ttyUSB3">/dev/ttyUSB3</option>
	<option selected value="/dev/ttyUSB4">/dev/ttyUSB4</option>
	<option value="/dev/ttyUSB5">/dev/ttyUSB5</option>
	<option value="/dev/ttyUSB6">/dev/ttyUSB6</option>
	<option value="/dev/ttyUSB7">/dev/ttyUSB7</option>
	<option value="/dev/ttyUSB8">/dev/ttyUSB8</option>
	<option value="/dev/ttyUSB9">/dev/ttyUSB9</option>\n';	
} elsif ($rs_connect eq "/dev/ttyUSB5") {
	$rs_connect_select = '
	<option value="/dev/ttyAMA0">/dev/ttyAMA0</option>
    <option value="/dev/ttyUSB0">/dev/ttyUSB0</option>
	<option value="/dev/ttyUSB1">/dev/ttyUSB1</option>
	<option value="/dev/ttyUSB2">/dev/ttyUSB2</option>
	<option value="/dev/ttyUSB3">/dev/ttyUSB3</option>
	<option value="/dev/ttyUSB4">/dev/ttyUSB4</option>
	<option selected value="/dev/ttyUSB5">/dev/ttyUSB5</option>
	<option value="/dev/ttyUSB6">/dev/ttyUSB6</option>
	<option value="/dev/ttyUSB7">/dev/ttyUSB7</option>
	<option value="/dev/ttyUSB8">/dev/ttyUSB8</option>
	<option value="/dev/ttyUSB9">/dev/ttyUSB9</option>\n';	
	} elsif ($rs_connect eq "/dev/ttyUSB6") {
	$rs_connect_select = '
	<option value="/dev/ttyAMA0">/dev/ttyAMA0</option>
    <option value="/dev/ttyUSB0">/dev/ttyUSB0</option>
	<option value="/dev/ttyUSB1">/dev/ttyUSB1</option>
	<option value="/dev/ttyUSB2">/dev/ttyUSB2</option>
	<option value="/dev/ttyUSB3">/dev/ttyUSB3</option>
	<option value="/dev/ttyUSB4">/dev/ttyUSB4</option>
	<option value="/dev/ttyUSB5">/dev/ttyUSB5</option>
	<option selected value="/dev/ttyUSB6">/dev/ttyUSB6</option>
	<option value="/dev/ttyUSB7">/dev/ttyUSB7</option>
	<option value="/dev/ttyUSB8">/dev/ttyUSB8</option>
	<option value="/dev/ttyUSB9">/dev/ttyUSB9</option>\n';	
	} elsif ($rs_connect eq "/dev/ttyUSB2") {
	$rs_connect_select = '
	<option value="/dev/ttyAMA0">/dev/ttyAMA0</option>
    <option value="/dev/ttyUSB0">/dev/ttyUSB0</option>
	<option value="/dev/ttyUSB1">/dev/ttyUSB1</option>
	<option value="/dev/ttyUSB2">/dev/ttyUSB2</option>
	<option value="/dev/ttyUSB3">/dev/ttyUSB3</option>
	<option value="/dev/ttyUSB4">/dev/ttyUSB4</option>
	<option value="/dev/ttyUSB5">/dev/ttyUSB5</option>
	<option value="/dev/ttyUSB6">/dev/ttyUSB6</option>
	<option selected value="/dev/ttyUSB7">/dev/ttyUSB7</option>
	<option value="/dev/ttyUSB8">/dev/ttyUSB8</option>
	<option value="/dev/ttyUSB9">/dev/ttyUSB9</option>\n';	
	} elsif ($rs_connect eq "/dev/ttyUSB2") {
	$rs_connect_select = '
	<option value="/dev/ttyAMA0">/dev/ttyAMA0</option>
    <option value="/dev/ttyUSB0">/dev/ttyUSB0</option>
	<option value="/dev/ttyUSB1">/dev/ttyUSB1</option>
	<option value="/dev/ttyUSB2">/dev/ttyUSB2</option>
	<option value="/dev/ttyUSB3">/dev/ttyUSB3</option>
	<option value="/dev/ttyUSB4">/dev/ttyUSB4</option>
	<option value="/dev/ttyUSB5">/dev/ttyUSB5</option>
	<option value="/dev/ttyUSB6">/dev/ttyUSB6</option>
	<option value="/dev/ttyUSB7">/dev/ttyUSB7</option>
	<option selected value="/dev/ttyUSB8">/dev/ttyUSB8</option>
	<option value="/dev/ttyUSB9">/dev/ttyUSB9</option>\n';	
	} elsif ($rs_connect eq "/dev/ttyUSB2") {
	$rs_connect_select = '
	<option value="/dev/ttyAMA0">/dev/ttyAMA0</option>
    <option value="/dev/ttyUSB0">/dev/ttyUSB0</option>
	<option value="/dev/ttyUSB1">/dev/ttyUSB1</option>
	<option value="/dev/ttyUSB2">/dev/ttyUSB2</option>
	<option value="/dev/ttyUSB3">/dev/ttyUSB3</option>
	<option value="/dev/ttyUSB4">/dev/ttyUSB4</option>
	<option value="/dev/ttyUSB5">/dev/ttyUSB5</option>
	<option value="/dev/ttyUSB6">/dev/ttyUSB6</option>
	<option value="/dev/ttyUSB7">/dev/ttyUSB7</option>
	<option value="/dev/ttyUSB8">/dev/ttyUSB8</option>
	<option selected value="/dev/ttyUSB9">/dev/ttyUSB9</option>\n';	
	} elsif ($rs_connect eq "/dev/ttyUSB2") {
	$rs_connect_select = '
	<option value="/dev/ttyAMA0">/dev/ttyAMA0</option>
    <option value="/dev/ttyUSB0">/dev/ttyUSB0</option>
	<option value="/dev/ttyUSB1">/dev/ttyUSB1</option>
	<option selected value="/dev/ttyUSB2">/dev/ttyUSB2</option>
	<option value="/dev/ttyUSB3">/dev/ttyUSB3</option>
	<option value="/dev/ttyUSB4">/dev/ttyUSB4</option>
	<option value="/dev/ttyUSB5">/dev/ttyUSB5</option>
	<option value="/dev/ttyUSB6">/dev/ttyUSB6</option>
	<option value="/dev/ttyUSB7">/dev/ttyUSB7</option>
	<option value="/dev/ttyUSB8">/dev/ttyUSB8</option>
	<option value="/dev/ttyUSB9">/dev/ttyUSB9</option>\n';	
	} else { 
	$rs_connect_select = '
	<option selected value="/dev/ttyAMA0">/dev/ttyAMA0</option>
    <option value="/dev/ttyUSB0">/dev/ttyUSB0</option>
	<option value="/dev/ttyUSB1">/dev/ttyUSB1</option>
	<option value="/dev/ttyUSB2">/dev/ttyUSB2</option>
	<option value="/dev/ttyUSB3">/dev/ttyUSB3</option>
	<option value="/dev/ttyUSB4">/dev/ttyUSB4</option>
	<option value="/dev/ttyUSB5">/dev/ttyUSB5</option>
	<option value="/dev/ttyUSB6">/dev/ttyUSB6</option>
	<option value="/dev/ttyUSB7">/dev/ttyUSB7</option>
	<option value="/dev/ttyUSB8">/dev/ttyUSB8</option>
	<option value="/dev/ttyUSB9">/dev/ttyUSB9</option>\n';
}


#Switches
if ($meter_active_1 eq "1") {
	$meter_active_1_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_1_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_2 eq "1") {
	$meter_active_2_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_2_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_3 eq "1") {
	$meter_active_3_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_3_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_4 eq "1") {
	$meter_active_4_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_4_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_5 eq "1") {
	$meter_active_5_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_5_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_6 eq "1") {
	$meter_active_6_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_6_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_7 eq "1") {
	$meter_active_7_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_7_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_8 eq "1") {
	$meter_active_8_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_8_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_9 eq "1") {
	$meter_active_9_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_9_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_10 eq "1") {
	$meter_active_10_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_10_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_11 eq "1") {
	$meter_active_11_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_11_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_12 eq "1") {
	$meter_active_12_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_12_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_13 eq "1") {
	$meter_active_13_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_13_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_14 eq "1") {
	$meter_active_14_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_14_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}
if ($meter_active_15 eq "1") {
	$meter_active_15_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_15_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}



if ($meter_active_14 eq "1") {
	$meter_active_14_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$meter_active_14_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}


#Debug Switch
if ($debug eq "1") {
	$debug_selected = '<option value="0">Aus</option><option value="1" selected>Ein</option>';
} else {
	$debug_selected = '<option value="0" selected>Aus</option><option value="1">Ein</option>';
}


    if ( !$header_already_sent ) { print "Content-Type: text/html\n\n"; }

    $template_title = "ModBus PowerMeter";

    # Print Template
    &lbheader;
    open( F,
        "$installfolder/templates/plugins/$psubfolder/$lang/settings.html" )
      || die "Missing template plugins/$psubfolder/$lang/settings.html";
    while (<F>) {
        $_ =~ s/<!--\$(.*?)-->/${$1}/g;
        print $_;
    }
    close(F);
    &footer;
    exit;
}

#####################################################
# Save-Sub
#####################################################

sub save {
	
	$pname = "modbus_powermeter"; 
    # Everything from Forms
    $meter_active_1 = param('meter_active_1');
	$id_1     = param('id_1');
    $time_1     = param('time_1');
    $meter_active_2 = param('meter_active_2');
	$id_2     = param('id_2');
    $time_2     = param('time_2');
    $meter_active_3 = param('meter_active_3');
	$id_3     = param('id_3');
    $time_3     = param('time_3');
    $meter_active_1 = param('meter_active_1');
	$id_4     = param('id_4');
    $time_4     = param('time_4');
    $meter_active_5 = param('meter_active_5');
	$id_5     = param('id_5');
    $time_5     = param('time_5');
    $meter_active_6 = param('meter_active_6');
	$id_6     = param('id_6');
    $time_6     = param('time_6');
    $meter_active_7 = param('meter_active_7');
	$id_7     = param('id_7');
    $time_7     = param('time_7');
    $meter_active_8 = param('meter_active_8');
	$id_8     = param('id_8');
    $time_8     = param('time_8');	
    $meter_active_9 = param('meter_active_9');
	$id_9     = param('id_9');
    $time_9     = param('time_9');
    $meter_active_10 = param('meter_active_10');
	$id_10    = param('id_10');
    $time_10    = param('time_10');
    $meter_active_11 = param('meter_active_11');
	$id_11    = param('id_11');
    $time_11    = param('time_11');
    $meter_active_12 = param('meter_active_12');
	$id_12    = param('id_12');
    $time_12    = param('time_12');
    $meter_active_13 = param('meter_active_13');
	$id_13    = param('id_13');
    $time_13    = param('time_13');
    $meter_active_14 = param('meter_active_14');
	$id_14    = param('id_14');
    $time_14    = param('time_14');
    $meter_active_15 = param('meter_active_15');
	$id_15    = param('id_15');
    $time_15    = param('time_15');	
	
	$httpport = param('httpport');
	$rs_connect = param('rs_connect');
	$rs_baud = param('rs_baud');
	
    $udpport = param('udpport');
	$miniserver = param('miniserver');
	$debug = param('debug');
	
	if ($meter_active_1 ne 1) { $meter_active_1 = 0 }
	if ($meter_active_2 ne 1) { $meter_active_2 = 0 }
	if ($meter_active_3 ne 1) { $meter_active_3 = 0 }
	if ($meter_active_4 ne 1) { $meter_active_4 = 0 }
	if ($meter_active_5 ne 1) { $meter_active_5 = 0 }
	if ($meter_active_6 ne 1) { $meter_active_6 = 0 }
	if ($meter_active_7 ne 1) { $meter_active_7 = 0 }
	if ($meter_active_8 ne 1) { $meter_active_8 = 0 }
	if ($meter_active_9 ne 1) { $meter_active_9 = 0 }
	if ($meter_active_10 ne 1) { $meter_active_10 = 0 }
	if ($meter_active_11 ne 1) { $meter_active_11 = 0 }
	if ($meter_active_12 ne 1) { $meter_active_12 = 0 }
	if ($meter_active_13 ne 1) { $meter_active_13= 0 }
	if ($meter_active_14 ne 1) { $meter_active_14 = 0 }
	if ($meter_active_15 ne 1) { $meter_active_15 = 0 }
	if ($debug ne 1) { $debug = 0 }
	

    # Write configuration file(s)
	$pcfg->param( "MINISERVER.MINISERVER", "MINISERVER$miniserver" );
    $pcfg->param( "MINISERVER.UDPPORT", "$udpport" );
	$pcfg->param( "DEBUG.DEBUG", "$debug" );
	
	$pcfg->param( "RS485.CONNECT", "$rs_connect" );
	$pcfg->param( "RS485.BAUD", "$rs_baud" );
	$pcfg->param( "HTTP.PORT", "$httpport" );	
	
	$pcfg->param( "METER_1.METER_ACTIVE", "$meter_active_1" );
    $pcfg->param( "METER_1.ID", "$id_1" );
    $pcfg->param( "METER_1.TIME", "$time_1" );
	$pcfg->param( "METER_2.METER_ACTIVE", "$meter_active_2" );
    $pcfg->param( "METER_2.ID", "$id_2" );
    $pcfg->param( "METER_2.TIME", "$time_2" );
	$pcfg->param( "METER_3.METER_ACTIVE", "$meter_active_3" );
    $pcfg->param( "METER_3.ID", "$id_3" );
    $pcfg->param( "METER_3.TIME", "$time_3" );
	$pcfg->param( "METER_4.METER_ACTIVE", "$meter_active_4" );
    $pcfg->param( "METER_4.ID", "$id_4" );
    $pcfg->param( "METER_4.TIME", "$time_4" );
	$pcfg->param( "METER_5.METER_ACTIVE", "$meter_active_5" );
    $pcfg->param( "METER_5.ID", "$id_5" );
    $pcfg->param( "METER_5.TIME", "$time_5" );
	$pcfg->param( "METER_6.METER_ACTIVE", "$meter_active_6" );
    $pcfg->param( "METER_6.ID", "$id_6" );
    $pcfg->param( "METER_6.TIME", "$time_6" );
	$pcfg->param( "METER_7.METER_ACTIVE", "$meter_active_7" );
    $pcfg->param( "METER_7.ID", "$id_7" );
    $pcfg->param( "METER_7.TIME", "$time_7" );	
	$pcfg->param( "METER_8.METER_ACTIVE", "$meter_active_8" );
    $pcfg->param( "METER_8.ID", "$id_8" );
    $pcfg->param( "METER_8.TIME", "$time_8" );
	$pcfg->param( "METER_9.METER_ACTIVE", "$meter_active_9" );
    $pcfg->param( "METER_9.ID", "$id_9" );
    $pcfg->param( "METER_9.TIME", "$time_9" );
	$pcfg->param( "METER_10.METER_ACTIVE", "$meter_active_10" );
    $pcfg->param( "METER_10.ID", "$id_10" );
    $pcfg->param( "METER_10.TIME", "$time_10" );	
	$pcfg->param( "METER_11.METER_ACTIVE", "$meter_active_11" );
    $pcfg->param( "METER_11.ID", "$id_11" );
    $pcfg->param( "METER_11.TIME", "$time_11" );
	$pcfg->param( "METER_12.METER_ACTIVE", "$meter_active_12" );
    $pcfg->param( "METER_12.ID", "$id_12" );
    $pcfg->param( "METER_12.TIME", "$time_12" );
	$pcfg->param( "METER_13.METER_ACTIVE", "$meter_active_13" );
    $pcfg->param( "METER_13.ID", "$id_13" );
    $pcfg->param( "METER_13.TIME", "$time_13" );
	$pcfg->param( "METER_14.METER_ACTIVE", "$meter_active_14" );
    $pcfg->param( "METER_14.ID", "$id_14" );
    $pcfg->param( "METER_14.TIME", "$time_14" );
	$pcfg->param( "METER_15.METER_ACTIVE", "$meter_active_15" );
    $pcfg->param( "METER_15.ID", "$id_15" );
    $pcfg->param( "METER_15.TIME", "$time_15" );
    $pcfg->save();	
     
	
    if ( !$header_already_sent ) { print "Content-Type: text/html\n\n"; }

    $template_title = "Modbus PowerMeter";
    $message = $phraseplugin->param("TXT0002");
    $nexturl = "./index.cgi?do=form";

    # Print Template
    &lbheader;
    open( F, "$installfolder/templates/system/$lang/success.html" )
      || die "Missing template system/$lang/succses.html";
    while (<F>) {
        $_ =~ s/<!--\$(.*?)-->/${$1}/g;
        print $_;
    }
    close(F);
    &footer;
    exit;
	
}

#####################################################
# UDP Test
#####################################################

sub test_udp {

	system("$installfolder/data/plugins/$psubfolder/reader.sh &");	
	
		if ( !$header_already_sent ) { print "Content-Type: text/html\n\n"; }
	$template_title = "Modbus PowerMeter";
    $message = $phraseplugin->param("TXT0003");
    $nexturl = "./index.cgi?do=form";
		
	# Print Template
    &lbheader;
    open( F, "$installfolder/templates/system/$lang/success.html" )
      || die "Missing template system/$lang/succses.html";
    while (<F>) {
        $_ =~ s/<!--\$(.*?)-->/${$1}/g;
        print $_;
    }
    close(F);
    &footer;
    exit;
}

#####################################################
# Page-Header-Sub
#####################################################

sub lbheader {

    # Create Help page
    $helplink = "http://www.loxwiki.eu:80/display/LOXBERRY/Miniserverbackup";
    open( F, "$installfolder/templates/plugins/$psubfolder/$lang/help.html" )
      || die "Missing template plugins/$psubfolder/$lang/help.html";
    @help = <F>;
    foreach (@help) {
        s/[\n\r]/ /g;
        $helptext = $helptext . $_;
    }
    close(F);
    open( F, "$installfolder/templates/system/$lang/header.html" )
      || die "Missing template system/$lang/header.html";
    while (<F>) {
        $_ =~ s/<!--\$(.*?)-->/${$1}/g;
        print $_;
    }
    close(F);
}

#####################################################
# Footer
#####################################################

sub footer {
    open( F, "$installfolder/templates/system/$lang/footer.html" )
      || die "Missing template system/$lang/footer.html";
    while (<F>) {
        $_ =~ s/<!--\$(.*?)-->/${$1}/g;
        print $_;
    }
    close(F);
}

